#include <string>
#include <vector>
#include <Wire.h>
#include "Camera.h"
#include <Arduino.h>
#include "LedPanel.h"
#include "utilities.h"
// #include "Adafruit_VL53L0X.h"

double Setpointx, Inputx, Outputx;
double Setpointy, Inputy, Outputy;

// Identify all the global constants that will be used by the robot
const int BAUD_RATE = 115200;
const int MAX_SPEED = 255;
const int SEEKING_SPEED = 70;
const double RESOLUTION_W = 320.0;
const double RESOLUTION_H = 240.0;
const int ENDDEMO_TAG = 0;
const uint32_t THISNODE_ID = 88789821;

//Identify all the variables that will be used by the robot
int findcolor = 1; //based on the same code from openMV. Determine the color you want to find
char newPTCL = '1';
int pau = 0;
int displayTracking = 0;
int8_t threshold[6] = {0, 0, 0, 0, 0, 0};
int BASE_SPEED = 50; //125;
int ball_capture = 0;

//The debug LEDs that we will be using. Description at:
const int DEBUG_STATE = 31;
const int DEBUG_KP = 30;
const int DEBUG_KI = 29;
const int DEBUG_KD = 28;
const int DEBUG_BASESPEED = 27;
const int DEBUG_THRESHOLD_MIN = 26;
const int DEBUG_THRESHOLD_MAX = 25;
const int DEBUG_VERTICALSPEED = 17;
const int DEBUG_RSPEED = 16;
const int DEBUG_LSPEED = 24;

//Create the interface that will be used by the camera
openmv::rpc_scratch_buffer<256> scratch_buffer; // All RPC objects share this buffer.
openmv::rpc_i2c_master interface(0x12, 100000); //to make this more robust, consider making address and rate as constructor argument

Camera cam(&interface);
LedPanel panel(NUMPIXELS,PIN_PIXELS);

void setup() {
  Serial.begin(BAUD_RATE);
  translateCodetoThreshold(findcolor);
  Wire.begin();
  AFMS.begin();  // create with the default frequency 1.6KHz
  interface.begin(); //communication between ESP and OpenMV
  panel.beginPanel();
    
  Setpointx = 160.0; 
  Setpointy = 120.0; //the values that the PID will try to reach
  // MeshSetup();
  PIDSetup();
  LocalizationSetup();
}

void loop() {  
  panel.singleLED(DEBUG_BASESPEED, BASE_SPEED, BASE_SPEED, BASE_SPEED);
  
  #ifdef MESH_CPP
  thisNode.update();
  #endif
  
  while(pau == 1){
    #ifdef MESH_CPP
      thisNode.update();
    #endif
    int zero = 0;
    moveVertical(zero);
    moveHorizontal(zero,zero);
    panel.singleLED(DEBUG_STATE, 10, 10, 0);
    Serial.println("pause");
  }
  
  int xtemp = 0;
  int ytemp = 0;
  int angletemp = 0;
  int x = 0;
  int y = 0;
  int id = -1;
  int tx = 0;
  int ty = 0;
  int tz = 0;
  int rx = 0;
  int ry = 0;
  int rz = 0;
  int8_t goal[3] = {0,1,2};
  panel.singleLED(DEBUG_STATE, 10, 0, 0); //standby
  
  altitudeBarrier();
  
  if(insideWindyArea){
    // leave windy area
    // flip horizontal inputs
  }
  
  // Have we captured the ball?
  // ball_capture = lidar_output()
  
  if(ball_capture){
	  if(cam.exe_goalfinder(goal[0], goal[1], goal[2], id, x, y, tz, rx, ry, rz)){
      if (displayTracking > 0){
		    displayTrackedObject(x, y, RESOLUTION_W, RESOLUTION_H); //THIS NEEDS WORK
      }
      panel.singleLED(DEBUG_STATE, 0, 10, 0);
      Serial.println("blob detected");
      Serial.print("x value: ");
      Serial.println(x);
      Serial.print("y value: ");
      Serial.println(y);
      Inputx = x/1.00;
      Inputy = y/1.00;
      PID_x.Compute();
      PID_y.Compute();
  
      Serial.print("Outputx: ");
      Serial.println(Outputx);
      Serial.print("Outputy: ");
      Serial.println(Outputy);
      
	  moveVertical(Outputy);
      moveHorizontal(Outputx, BASE_SPEED);
    } else { //seeking algorithm
      //panel.resetPanel();
      panel.singleLED(DEBUG_STATE, 10, 10, 10);
      Serial.println("seeking...");
      int zero = 0;
      moveVertical(zero);
      moveHorizontal(SEEKING_SPEED, zero);
    }
  }
  else {
	  ball_detect = cam.exe_color_detection_biggestblob(threshold[0], threshold[1], threshold[2], threshold[3], threshold[4], threshold[5], x, y);
    if(ball_detect) {
      // move towards ball
      
    }
    else {
      // seek the ball
      // add seeking algorithm - turn around for 360 degrees?
    }
  }
}

void altitudeBarrier(){
  if(altitude < lowHeightBarrier){
    moveVertical((lowHeightBarrier - altitude)*255/lowHeightBarrier);
  }
  else if(altitude > ceilingHeight - highHeightBarrier){
    moveVertical(((ceilingHeight - highHeightBarrier) - altitude)*255/highHeightBarrier);
  }	
}

